@@ -9,11 +9,12 @@
 #define __DEBUG_XML__
 #include "tree.h"
 
-extern void xmlDebugDumpString(FILE *output, const CHAR *str);
+extern void xmlDebugDumpString(FILE *output, const xmlChar *str);
 extern void xmlDebugDumpAttr(FILE *output, xmlAttrPtr attr, int depth);
 extern void xmlDebugDumpAttrList(FILE *output, xmlAttrPtr attr, int depth);
 extern void xmlDebugDumpOneNode(FILE *output, xmlNodePtr node, int depth);
 extern void xmlDebugDumpNode(FILE *output, xmlNodePtr node, int depth);
 extern void xmlDebugDumpNodeList(FILE *output, xmlNodePtr node, int depth);
 extern void xmlDebugDumpDocument(FILE *output, xmlDocPtr doc);
+extern void xmlDebugDumpEntities(FILE *output, xmlDocPtr doc);
 #endif /* __DEBUG_XML__ */
