@@ -38,6 +38,7 @@
 #include "debugXML.h"
 
 static int debug = 0;
+static int debugent = 0;
 static int copy = 0;
 static int recovery = 0;
 static int noent = 0;
@@ -45,6 +46,7 @@ static int noout = 0;
 static int valid = 0;
 static int repeat = 0;
 static int insert = 0;
+static int compress = 0;
 
 extern int xmlDoValidityCheckingDefaultValue;
 
@@ -168,11 +170,16 @@ void parseAndPrintFile(char *filename) {
 	/*
 	 * print it.
 	 */
-	if (!debug)
-	    xmlDocDump(stdout, doc);
-	else
+	if (!debug) {
+	    if (compress)
+		xmlSaveFile("-", doc);
+	    else
+		xmlDocDump(stdout, doc);
+	} else
 	    xmlDebugDumpDocument(stdout, doc);
     }
+    if (debugent)	
+	xmlDebugDumpEntities(stdout, doc);
 
     /*
      * free it.
@@ -203,10 +210,15 @@ void parseAndPrintBuffer(xmlChar *buf) {
     /*
      * print it.
      */
-    if (!debug)
-	xmlDocDump(stdout, doc);
-    else
+    if (!debug) {
+        if (compress)
+	    xmlSaveFile("-", doc);
+	else
+	    xmlDocDump(stdout, doc);
+    } else
         xmlDebugDumpDocument(stdout, doc);
+    if (debugent)	
+        xmlDebugDumpEntities(stdout, doc);
 
     /*
      * free it.
@@ -221,6 +233,8 @@ int main(int argc, char **argv) {
     for (i = 1; i < argc ; i++) {
 	if ((!strcmp(argv[i], "-debug")) || (!strcmp(argv[i], "--debug")))
 	    debug++;
+	if ((!strcmp(argv[i], "-debugent")) || (!strcmp(argv[i], "--debugent")))
+	    debugent++;
 	else if ((!strcmp(argv[i], "-copy")) || (!strcmp(argv[i], "--copy")))
 	    copy++;
 	else if ((!strcmp(argv[i], "-recover")) ||
@@ -241,6 +255,11 @@ int main(int argc, char **argv) {
 	else if ((!strcmp(argv[i], "-repeat")) ||
 	         (!strcmp(argv[i], "--repeat")))
 	    repeat++;
+	else if ((!strcmp(argv[i], "-compress")) ||
+	         (!strcmp(argv[i], "--compress"))) {
+	    compress++;
+	    xmlSetCompressMode(9);
+        }
     }
     if (noent != 0) xmlSubstituteEntitiesDefault(1);
     if (valid != 0) xmlDoValidityCheckingDefaultValue = 1;
@@ -255,10 +274,11 @@ int main(int argc, char **argv) {
 	}
     }
     if (files == 0) {
-	printf("Usage : %s [--debug] [--copy] [--recover] [--noent] [--noout] [--valid] [--repeat] XMLfiles ...\n",
+	printf("Usage : %s [--debug] [--debugent] [--copy] [--recover] [--noent] [--noout] [--valid] [--repeat] XMLfiles ...\n",
 	       argv[0]);
 	printf("\tParse the XML files and output the result of the parsing\n");
 	printf("\t--debug : dump a debug tree of the in-memory document\n");
+	printf("\t--debugent : debug the entities defined in the document\n");
 	printf("\t--copy : used to test the internal copy implementation\n");
 	printf("\t--recover : output what is parsable on broken XmL documents\n");
 	printf("\t--noent : substitute entity references by their value\n");
@@ -266,7 +286,9 @@ int main(int argc, char **argv) {
 	printf("\t--valid : validate the document in addition to std well-formed check\n");
 	printf("\t--repeat : parse the file 100 times, for timing or profiling\n");
 	printf("\t--insert : test for valid insertions\n");
+	printf("\t--compress : turn on gzip compression of output\n");
     }
+    xmlCleanupParser();
     xmlMemoryDump();
 
     return(0);
