@@ -34,8 +34,10 @@
 static xmlChar xmlStringText[] = { 't', 'e', 'x', 't', 0 };
 int oldXMLWDcompatibility = 0;
 int xmlIndentTreeOutput = 1;
+xmlBufferAllocationScheme xmlBufferAllocScheme = XML_BUFFER_ALLOC_EXACT;
 
 static int xmlCompressMode = 0;
+static int xmlCheckDTD = 1;
 
 #define UPDATE_LAST_CHILD(n) if ((n) != NULL) {				\
     xmlNodePtr ulccur = (n)->childs;					\
@@ -52,6 +54,35 @@ static int xmlCompressMode = 0;
  *									*
  ************************************************************************/
  
+/**
+ * xmlSetBufferAllocationScheme:
+ * @scheme:  allocation method to use
+ * 
+ * Set the buffer allocation method.  Types are
+ * XML_BUFFER_ALLOC_EXACT - use exact sizes, keeps memory usage down
+ * XML_BUFFER_ALLOC_DOUBLEIT - double buffer when extra needed, 
+ *                             improves performance
+ */
+void
+xmlSetBufferAllocationScheme(xmlBufferAllocationScheme scheme) {
+    xmlBufferAllocScheme = scheme;
+}
+
+/**
+ * xmlGetBufferAllocationScheme:
+ *
+ * Types are
+ * XML_BUFFER_ALLOC_EXACT - use exact sizes, keeps memory usage down
+ * XML_BUFFER_ALLOC_DOUBLEIT - double buffer when extra needed, 
+ *                             improves performance
+ * 
+ * Returns the current allocation scheme
+ */
+xmlBufferAllocationScheme
+xmlGetBufferAllocationScheme() {
+    return xmlBufferAllocScheme;
+}
+
 /**
  * xmlUpgradeOldNs:
  * @doc:  a document pointer
@@ -403,7 +434,7 @@ xmlNewDoc(const xmlChar *version) {
     cur->oldNs = NULL;
     cur->encoding = NULL;
     cur->standalone = -1;
-    cur->compression = xmlCompressMode;
+    cur->compression = -1; /* not initialized */
     cur->ids = NULL;
     cur->refs = NULL;
 #ifndef XML_WITHOUT_CORBA
@@ -684,12 +715,21 @@ xmlNodeListGetString(xmlDocPtr doc, xmlNodePtr list, int inLine) {
 
     while (node != NULL) {
         if (node->type == XML_TEXT_NODE) {
-	    if ((inLine) || (doc->type == XML_HTML_DOCUMENT_NODE))
+	    if ((inLine) || (doc->type == XML_HTML_DOCUMENT_NODE)) {
+#ifndef XML_USE_BUFFER_CONTENT
 		ret = xmlStrcat(ret, node->content);
-	    else {
+#else
+		ret = xmlStrcat(ret, xmlBufferContent(node->content));
+#endif
+	    } else {
 	        xmlChar *buffer;
 
+#ifndef XML_USE_BUFFER_CONTENT
 		buffer = xmlEncodeEntitiesReentrant(doc, node->content);
+#else
+		buffer = xmlEncodeEntitiesReentrant(doc,
+					    xmlBufferContent(node->content));
+#endif
 		if (buffer != NULL) {
 		    ret = xmlStrcat(ret, buffer);
 		    xmlFree(buffer);
@@ -700,8 +740,13 @@ xmlNodeListGetString(xmlDocPtr doc, xmlNodePtr list, int inLine) {
 		ent = xmlGetDocEntity(doc, node->name);
 		if (ent != NULL)
 		    ret = xmlStrcat(ret, ent->content);
-		else
+		else {
+#ifndef XML_USE_BUFFER_CONTENT
 		    ret = xmlStrcat(ret, node->content);
+#else
+		    ret = xmlStrcat(ret, xmlBufferContent(node->content));
+#endif
+		}    
             } else {
 	        xmlChar buf[2];
 		buf[0] = '&'; buf[1] = 0;
@@ -753,8 +798,12 @@ xmlNewProp(xmlNodePtr node, const xmlChar *name, const xmlChar *value) {
     cur->node = node; 
     cur->ns = NULL;
     cur->name = xmlStrdup(name);
-    if (value != NULL)
-	cur->val = xmlStringGetNodeList(node->doc, value);
+    if (value != NULL) {
+	xmlChar *buffer;
+	buffer = xmlEncodeEntitiesReentrant(node->doc, value);
+	cur->val = xmlStringGetNodeList(node->doc, buffer);
+	xmlFree(buffer);
+    }	
     else 
 	cur->val = NULL;
 #ifndef XML_WITHOUT_CORBA
@@ -960,9 +1009,16 @@ xmlNewPI(const xmlChar *name, const xmlChar *content) {
     cur->name = xmlStrdup(name);
     cur->ns = NULL;
     cur->nsDef = NULL;
-    if (content != NULL)
+    if (content != NULL) {
+#ifndef XML_USE_BUFFER_CONTENT
 	cur->content = xmlStrdup(content);
-    else
+#else
+	cur->content = xmlBufferCreateSize(0);
+	xmlBufferSetAllocationScheme(cur->content,
+		                     xmlGetBufferAllocationScheme());
+	xmlBufferAdd(cur->content, content, -1);
+#endif
+    } else
 	cur->content = NULL;
 #ifndef XML_WITHOUT_CORBA
     cur->_private = NULL;
@@ -1023,15 +1079,20 @@ xmlNewNode(xmlNsPtr ns, const xmlChar *name) {
  * @doc:  the document
  * @ns:  namespace if any
  * @name:  the node name
- * @content:  the text content if any
+ * @content:  the XML text content if any
  *
  * Creation of a new node element within a document. @ns and @content
  * are optionnal (NULL).
+ * NOTE: @content is supposed to be a piece of XML CDATA, so it allow entities
+ *       references, but XML special chars need to be escaped first by using
+ *       xmlEncodeEntitiesReentrant(). Use xmlNewDocRawNode() if you don't
+ *       need entities support.
+ *
  * Returns a pointer to the new node object.
  */
 xmlNodePtr
 xmlNewDocNode(xmlDocPtr doc, xmlNsPtr ns,
-                         const xmlChar *name, const xmlChar *content) {
+              const xmlChar *name, const xmlChar *content) {
     xmlNodePtr cur;
 
     cur = xmlNewNode(ns, name);
@@ -1046,6 +1107,35 @@ xmlNewDocNode(xmlDocPtr doc, xmlNsPtr ns,
 }
 
 
+/**
+ * xmlNewDocRawNode:
+ * @doc:  the document
+ * @ns:  namespace if any
+ * @name:  the node name
+ * @content:  the text content if any
+ *
+ * Creation of a new node element within a document. @ns and @content
+ * are optionnal (NULL).
+ *
+ * Returns a pointer to the new node object.
+ */
+xmlNodePtr
+xmlNewDocRawNode(xmlDocPtr doc, xmlNsPtr ns,
+                 const xmlChar *name, const xmlChar *content) {
+    xmlNodePtr cur;
+
+    cur = xmlNewNode(ns, name);
+    if (cur != NULL) {
+        cur->doc = doc;
+	if (content != NULL) {
+	    cur->childs = xmlNewDocText(doc, content);
+	    UPDATE_LAST_CHILD(cur)
+	}
+    }
+    return(cur);
+}
+
+
 /**
  * xmlNewText:
  * @content:  the text content
@@ -1078,9 +1168,16 @@ xmlNewText(const xmlChar *content) {
     cur->name = xmlStrdup(xmlStringText);
     cur->ns = NULL;
     cur->nsDef = NULL;
-    if (content != NULL)
+    if (content != NULL) {
+#ifndef XML_USE_BUFFER_CONTENT
 	cur->content = xmlStrdup(content);
-    else 
+#else
+	cur->content = xmlBufferCreateSize(0);
+	xmlBufferSetAllocationScheme(cur->content,
+		                     xmlGetBufferAllocationScheme());
+	xmlBufferAdd(cur->content, content, -1);
+#endif
+    } else 
 	cur->content = NULL;
 #ifndef XML_WITHOUT_CORBA
     cur->_private = NULL;
@@ -1089,6 +1186,62 @@ xmlNewText(const xmlChar *content) {
     return(cur);
 }
 
+/**
+ * xmlNewTextChild:
+ * @parent:  the parent node
+ * @ns:  a namespace if any
+ * @name:  the name of the child
+ * @content:  the text content of the child if any.
+ *
+ * Creation of a new child element, added at the end of @parent childs list.
+ * @ns and @content parameters are optionnal (NULL). If content is non NULL,
+ * a child TEXT node will be created containing the string content.
+ *
+ * Returns a pointer to the new node object.
+ */
+xmlNodePtr
+xmlNewTextChild(xmlNodePtr parent, xmlNsPtr ns,
+            const xmlChar *name, const xmlChar *content) {
+    xmlNodePtr cur, prev;
+
+    if (parent == NULL) {
+        fprintf(stderr, "xmlNewTextChild : parent == NULL\n");
+	return(NULL);
+    }
+
+    if (name == NULL) {
+        fprintf(stderr, "xmlNewTextChild : name == NULL\n");
+	return(NULL);
+    }
+
+    /*
+     * Allocate a new node
+     */
+    if (ns == NULL)
+	cur = xmlNewDocRawNode(parent->doc, parent->ns, name, content);
+    else
+	cur = xmlNewDocRawNode(parent->doc, ns, name, content);
+    if (cur == NULL) return(NULL);
+
+    /*
+     * add the new element at the end of the childs list.
+     */
+    cur->type = XML_ELEMENT_NODE;
+    cur->parent = parent;
+    cur->doc = parent->doc;
+    if (parent->childs == NULL) {
+        parent->childs = cur;
+	parent->last = cur;
+    } else {
+        prev = parent->last;
+	prev->next = cur;
+	cur->prev = prev;
+	parent->last = cur;
+    }
+
+    return(cur);
+}
+
 /**
  * xmlNewReference:
  * @doc: the document
@@ -1133,9 +1286,22 @@ xmlNewReference(xmlDocPtr doc, const xmlChar *name) {
     cur->nsDef = NULL;
 
     ent = xmlGetDocEntity(doc, cur->name);
-    if (ent != NULL)
+    if (ent != NULL) {
+#ifndef XML_USE_BUFFER_CONTENT
 	cur->content = ent->content;
-    else
+#else
+	/*
+	 * CJN 11.18.99 this might be a problem, since the xmlBuffer gets
+	 * a copy of this pointer.  Let's hope we don't manipulate it
+	 * later 
+	 */
+	cur->content = xmlBufferCreateSize(0);
+	xmlBufferSetAllocationScheme(cur->content,
+		                     xmlGetBufferAllocationScheme());
+	if (ent->content != NULL)
+	    xmlBufferAdd(cur->content, ent->content, -1);
+#endif
+    } else
         cur->content = NULL;
 #ifndef XML_WITHOUT_CORBA
     cur->_private = NULL;
@@ -1194,9 +1360,16 @@ xmlNewTextLen(const xmlChar *content, int len) {
     cur->name = xmlStrdup(xmlStringText);
     cur->ns = NULL;
     cur->nsDef = NULL;
-    if (content != NULL)
+    if (content != NULL) {
+#ifndef XML_USE_BUFFER_CONTENT
 	cur->content = xmlStrndup(content, len);
-    else 
+#else
+	cur->content = xmlBufferCreateSize(len);
+	xmlBufferSetAllocationScheme(cur->content,
+		                     xmlGetBufferAllocationScheme());
+	xmlBufferAdd(cur->content, content, len);
+#endif
+    } else 
 	cur->content = NULL;
 #ifndef XML_WITHOUT_CORBA
     cur->_private = NULL;
@@ -1256,9 +1429,16 @@ xmlNewComment(const xmlChar *content) {
     cur->name = xmlStrdup(xmlStringText);
     cur->ns = NULL;
     cur->nsDef = NULL;
-    if (content != NULL)
+    if (content != NULL) {
+#ifndef XML_USE_BUFFER_CONTENT
 	cur->content = xmlStrdup(content);
-    else 
+#else
+	cur->content = xmlBufferCreateSize(0);
+	xmlBufferSetAllocationScheme(cur->content,
+		                     xmlGetBufferAllocationScheme());
+	xmlBufferAdd(cur->content, content, -1);
+#endif
+    } else 
 	cur->content = NULL;
 #ifndef XML_WITHOUT_CORBA
     cur->_private = NULL;
@@ -1300,8 +1480,15 @@ xmlNewCDataBlock(xmlDocPtr doc, const xmlChar *content, int len) {
     cur->name = xmlStrdup(xmlStringText);
     cur->ns = NULL;
     cur->nsDef = NULL;
-    if ((content != NULL) && (len > 0)) {
+    if (content != NULL) {
+#ifndef XML_USE_BUFFER_CONTENT
 	cur->content = xmlStrndup(content, len);
+#else
+	cur->content = xmlBufferCreateSize(len);
+	xmlBufferSetAllocationScheme(cur->content,
+		                     xmlGetBufferAllocationScheme());
+	xmlBufferAdd(cur->content, content, len);
+#endif
     } else 
 	cur->content = NULL;
 #ifndef XML_WITHOUT_CORBA
@@ -1328,22 +1515,27 @@ xmlNewDocComment(xmlDocPtr doc, const xmlChar *content) {
     return(cur);
 }
 
+
 /**
  * xmlNewChild:
  * @parent:  the parent node
  * @ns:  a namespace if any
  * @name:  the name of the child
- * @content:  the content of the child if any.
+ * @content:  the XML content of the child if any.
  *
- * 
  * Creation of a new child element, added at the end of @parent childs list.
  * @ns and @content parameters are optionnal (NULL). If content is non NULL,
  * a child list containing the TEXTs and ENTITY_REFs node will be created.
+ * NOTE: @content is supposed to be a piece of XML CDATA, so it allow entities
+ *       references, but XML special chars need to be escaped first by using
+ *       xmlEncodeEntitiesReentrant(). Use xmlNewTextChild() if entities
+ *       support is not needed.
+ *
  * Returns a pointer to the new node object.
  */
 xmlNodePtr
 xmlNewChild(xmlNodePtr parent, xmlNsPtr ns,
-                       const xmlChar *name, const xmlChar *content) {
+            const xmlChar *name, const xmlChar *content) {
     xmlNodePtr cur, prev;
 
     if (parent == NULL) {
@@ -1441,12 +1633,12 @@ xmlAddChild(xmlNodePtr parent, xmlNodePtr cur) {
     xmlNodePtr prev;
 
     if (parent == NULL) {
-        fprintf(stderr, "xmladdChild : parent == NULL\n");
+        fprintf(stderr, "xmlAddChild : parent == NULL\n");
 	return(NULL);
     }
 
     if (cur == NULL) {
-        fprintf(stderr, "xmladdChild : child == NULL\n");
+        fprintf(stderr, "xmlAddChild : child == NULL\n");
 	return(NULL);
     }
 
@@ -1468,14 +1660,22 @@ xmlAddChild(xmlNodePtr parent, xmlNodePtr cur) {
     if (parent->content != NULL) {
         xmlNodePtr text;
 	
+#ifndef XML_USE_BUFFER_CONTENT
 	text = xmlNewDocText(parent->doc, parent->content);
+#else
+	text = xmlNewDocText(parent->doc, xmlBufferContent(parent->content));
+#endif
 	if (text != NULL) {
 	    text->next = parent->childs;
 	    if (text->next != NULL)
 		text->next->prev = text;
 	    parent->childs = text;
 	    UPDATE_LAST_CHILD(parent)
+#ifndef XML_USE_BUFFER_CONTENT
 	    xmlFree(parent->content);
+#else
+	    xmlBufferFree(parent->content);
+#endif
 	    parent->content = NULL;
 	}
     }
@@ -1548,7 +1748,11 @@ xmlFreeNode(xmlNodePtr cur) {
     if (cur->childs != NULL) xmlFreeNodeList(cur->childs);
     if (cur->properties != NULL) xmlFreePropList(cur->properties);
     if (cur->type != XML_ENTITY_REF_NODE)
+#ifndef XML_USE_BUFFER_CONTENT
 	if (cur->content != NULL) xmlFree(cur->content);
+#else
+    	if (cur->content != NULL) xmlBufferFree(cur->content);
+#endif
     if (cur->name != NULL) xmlFree((char *) cur->name);
     if (cur->nsDef != NULL) xmlFreeNsList(cur->nsDef);
     memset(cur, -1, sizeof(xmlNode));
@@ -1699,7 +1903,7 @@ xmlCopyPropList(xmlNodePtr target, xmlAttrPtr cur) {
 }
 
 /*
- * NOTE about the CopyNode operations !
+ * NOTE abeut the CopyNode operations !
  *
  * They are splitted into external and internal parts for one
  * tricky reason: namespaces. Doing a direct copy of a node
@@ -1745,9 +1949,18 @@ xmlStaticCopyNode(xmlNodePtr node, xmlDocPtr doc, xmlNodePtr parent,
         ret->name = NULL;
     ret->ns = NULL;
     ret->nsDef = NULL;
-    if ((node->content != NULL) && (node->type != XML_ENTITY_REF_NODE))
+    if ((node->content != NULL) && (node->type != XML_ENTITY_REF_NODE)) {
+#ifndef XML_USE_BUFFER_CONTENT
 	ret->content = xmlStrdup(node->content);
-    else
+#else
+	ret->content = xmlBufferCreateSize(xmlBufferLength(node->content));
+	xmlBufferSetAllocationScheme(ret->content,
+		                     xmlGetBufferAllocationScheme());
+	xmlBufferAdd(ret->content,
+		     xmlBufferContent(node->content),
+		     xmlBufferLength(node->content));
+#endif
+    } else
 	ret->content = NULL;
 #ifndef XML_WITHOUT_CORBA
     ret->_private = NULL;
@@ -1964,10 +2177,11 @@ xmlNodeSetLang(xmlNodePtr cur, const xmlChar *lang) {
  * attribute or the one carried by the nearest ancestor.
  *
  * Returns a pointer to the lang value, or NULL if not found
+ *     It's up to the caller to free the memory.
  */
-const xmlChar *
+xmlChar *
 xmlNodeGetLang(xmlNodePtr cur) {
-    const xmlChar *lang;
+    xmlChar *lang;
 
     while (cur != NULL) {
         lang = xmlGetProp(cur, BAD_CAST "xml:lang");
@@ -1978,6 +2192,59 @@ xmlNodeGetLang(xmlNodePtr cur) {
     return(NULL);
 }
  
+/**
+ * xmlNodeGetBase:
+ * @doc:  the document the node pertains to
+ * @cur:  the node being checked
+ *
+ * Searches for the BASE URL. The code should work on both XML
+ * and HTML document even if base mechanisms are completely different.
+ *
+ * Returns a pointer to the base URL, or NULL if not found
+ *     It's up to the caller to free the memory.
+ */
+xmlChar *
+xmlNodeGetBase(xmlDocPtr doc, xmlNodePtr cur) {
+    xmlChar *base;
+
+    if ((cur == NULL) && (doc == NULL)) 
+        return(NULL);
+    if (doc == NULL) doc = cur->doc;	
+    if ((doc != NULL) && (doc->type == XML_HTML_DOCUMENT_NODE)) {
+        cur = doc->root;
+	while ((cur != NULL) && (cur->name != NULL)) {
+	    if (cur->type != XML_ELEMENT_NODE) {
+	        cur = cur->next;
+		continue;
+	    }
+	    if ((!xmlStrcmp(cur->name, BAD_CAST "html")) ||
+	        (!xmlStrcmp(cur->name, BAD_CAST "HTML"))) {
+	        cur = cur->childs;
+		continue;
+	    }
+	    if ((!xmlStrcmp(cur->name, BAD_CAST "head")) ||
+	        (!xmlStrcmp(cur->name, BAD_CAST "HEAD"))) {
+	        cur = cur->childs;
+		continue;
+	    }
+	    if ((!xmlStrcmp(cur->name, BAD_CAST "base")) ||
+	        (!xmlStrcmp(cur->name, BAD_CAST "BASE"))) {
+                base = xmlGetProp(cur, BAD_CAST "href");
+		if (base != NULL) return(base);
+                return(xmlGetProp(cur, BAD_CAST "HREF"));
+	    }
+	}
+	return(NULL);
+    }
+    while (cur != NULL) {
+        base = xmlGetProp(cur, BAD_CAST "xml:base");
+	if (base != NULL)
+	    return(base);
+	cur = cur->parent;
+    }
+    return(NULL);
+}
+ 
 /**
  * xmlNodeGetContent:
  * @cur:  the node being read
@@ -2007,7 +2274,11 @@ xmlNodeGetContent(xmlNodePtr cur) {
 	}
         case XML_PI_NODE:
 	    if (cur->content != NULL)
+#ifndef XML_USE_BUFFER_CONTENT
 	        return(xmlStrdup(cur->content));
+#else
+	        return(xmlStrdup(xmlBufferContent(cur->content)));
+#endif
 	    return(NULL);
         case XML_ENTITY_REF_NODE:
         case XML_ENTITY_NODE:
@@ -2020,7 +2291,11 @@ xmlNodeGetContent(xmlNodePtr cur) {
         case XML_CDATA_SECTION_NODE:
         case XML_TEXT_NODE:
 	    if (cur->content != NULL)
+#ifndef XML_USE_BUFFER_CONTENT
 		return(xmlStrdup(cur->content));
+#else
+	        return(xmlStrdup(xmlBufferContent(cur->content)));
+#endif
             return(NULL);
     }
     return(NULL);
@@ -2043,7 +2318,11 @@ xmlNodeSetContent(xmlNodePtr cur, const xmlChar *content) {
         case XML_DOCUMENT_FRAG_NODE:
         case XML_ELEMENT_NODE:
 	    if (cur->content != NULL) {
+#ifndef XML_USE_BUFFER_CONTENT
 	        xmlFree(cur->content);
+#else
+		xmlBufferFree(cur->content);
+#endif
 		cur->content = NULL;
 	    }
 	    if (cur->childs != NULL) xmlFreeNodeList(cur->childs);
@@ -2058,12 +2337,25 @@ xmlNodeSetContent(xmlNodePtr cur, const xmlChar *content) {
         case XML_ENTITY_NODE:
         case XML_PI_NODE:
         case XML_COMMENT_NODE:
-	    if (cur->content != NULL) xmlFree(cur->content);
+	    if (cur->content != NULL) {
+#ifndef XML_USE_BUFFER_CONTENT
+		xmlFree(cur->content);
+#else
+	        xmlBufferFree(cur->content);
+#endif
+	    }	
 	    if (cur->childs != NULL) xmlFreeNodeList(cur->childs);
 	    cur->last = cur->childs = NULL;
-	    if (content != NULL)
+	    if (content != NULL) {
+#ifndef XML_USE_BUFFER_CONTENT
 		cur->content = xmlStrdup(content);
-	    else 
+#else
+		cur->content = xmlBufferCreateSize(0);
+		xmlBufferSetAllocationScheme(cur->content,
+			                     xmlGetBufferAllocationScheme());
+		xmlBufferAdd(cur->content, content, -1);
+#endif
+	    } else 
 		cur->content = NULL;
 	    break;
         case XML_DOCUMENT_NODE:
@@ -2093,7 +2385,11 @@ xmlNodeSetContentLen(xmlNodePtr cur, const xmlChar *content, int len) {
         case XML_DOCUMENT_FRAG_NODE:
         case XML_ELEMENT_NODE:
 	    if (cur->content != NULL) {
+#ifndef XML_USE_BUFFER_CONTENT
 	        xmlFree(cur->content);
+#else
+		xmlBufferFree(cur->content);
+#endif
 		cur->content = NULL;
 	    }
 	    if (cur->childs != NULL) xmlFreeNodeList(cur->childs);
@@ -2108,27 +2404,32 @@ xmlNodeSetContentLen(xmlNodePtr cur, const xmlChar *content, int len) {
         case XML_ENTITY_NODE:
         case XML_PI_NODE:
         case XML_COMMENT_NODE:
-	    if (cur->content != NULL) xmlFree(cur->content);
+        case XML_NOTATION_NODE:
+	    if (cur->content != NULL) {
+#ifndef XML_USE_BUFFER_CONTENT
+		xmlFree(cur->content);
+#else
+	        xmlBufferFree(cur->content);
+#endif
+	    }	
 	    if (cur->childs != NULL) xmlFreeNodeList(cur->childs);
 	    cur->childs = cur->last = NULL;
-	    if (content != NULL)
+	    if (content != NULL) {
+#ifndef XML_USE_BUFFER_CONTENT
 		cur->content = xmlStrndup(content, len);
-	    else 
+#else
+		cur->content = xmlBufferCreateSize(len);
+		xmlBufferSetAllocationScheme(cur->content,
+			                     xmlGetBufferAllocationScheme());
+		xmlBufferAdd(cur->content, content, len);
+#endif
+	    } else 
 		cur->content = NULL;
 	    break;
         case XML_DOCUMENT_NODE:
         case XML_HTML_DOCUMENT_NODE:
         case XML_DOCUMENT_TYPE_NODE:
 	    break;
-        case XML_NOTATION_NODE:
-	    if (cur->content != NULL) xmlFree(cur->content);
-	    if (cur->childs != NULL) xmlFreeNodeList(cur->childs);
-	    cur->childs = cur->last = NULL;
-	    if (content != NULL)
-		cur->content = xmlStrndup(content, len);
-	    else 
-		cur->content = NULL;
-	    break;
     }
 }
 
@@ -2156,9 +2457,18 @@ xmlNodeAddContentLen(xmlNodePtr cur, const xmlChar *content, int len) {
 		last = cur->last;
 	    } else {
 	        if (cur->content != NULL) {
+#ifndef XML_USE_BUFFER_CONTENT
 		    cur->childs = xmlStringGetNodeList(cur->doc, cur->content);
+#else
+		    cur->childs = xmlStringGetNodeList(cur->doc,
+			                       xmlBufferContent(cur->content));
+#endif
 		    UPDATE_LAST_CHILD(cur)
+#ifndef XML_USE_BUFFER_CONTENT
 		    xmlFree(cur->content);
+#else
+		    xmlBufferFree(cur->content);
+#endif
 		    cur->content = NULL;
 		    last = cur->last;
 		}
@@ -2180,16 +2490,18 @@ xmlNodeAddContentLen(xmlNodePtr cur, const xmlChar *content, int len) {
         case XML_ENTITY_NODE:
         case XML_PI_NODE:
         case XML_COMMENT_NODE:
-	    if (content != NULL)
+        case XML_NOTATION_NODE:
+	    if (content != NULL) {
+#ifndef XML_USE_BUFFER_CONTENT
 		cur->content = xmlStrncat(cur->content, content, len);
+#else
+		xmlBufferAdd(cur->content, content, len);
+#endif
+            }
         case XML_DOCUMENT_NODE:
         case XML_HTML_DOCUMENT_NODE:
         case XML_DOCUMENT_TYPE_NODE:
 	    break;
-        case XML_NOTATION_NODE:
-	    if (content != NULL)
-		cur->content = xmlStrncat(cur->content, content, len);
-	    break;
     }
 }
 
@@ -2227,7 +2539,11 @@ xmlTextMerge(xmlNodePtr first, xmlNodePtr second) {
     if (second == NULL) return(first);
     if (first->type != XML_TEXT_NODE) return(first);
     if (second->type != XML_TEXT_NODE) return(first);
+#ifndef XML_USE_BUFFER_CONTENT
     xmlNodeAddContent(first, second->content);
+#else
+    xmlNodeAddContent(first, xmlBufferContent(second->content));
+#endif
     xmlUnlinkNode(second);
     xmlFreeNode(second);
     return(first);
@@ -2303,6 +2619,7 @@ xmlNsPtr
 xmlSearchNs(xmlDocPtr doc, xmlNodePtr node, const xmlChar *nameSpace) {
     xmlNsPtr cur;
 
+    if ((node == NULL) || (nameSpace == NULL)) return(NULL);
     while (node != NULL) {
 	cur = node->nsDef;
 	while (cur != NULL) {
@@ -2315,6 +2632,8 @@ xmlSearchNs(xmlDocPtr doc, xmlNodePtr node, const xmlChar *nameSpace) {
 	}
 	node = node->parent;
     }
+#if 0    
+    /* Removed support for old namespaces */
     if (doc != NULL) {
         cur = doc->oldNs;
 	while (cur != NULL) {
@@ -2324,6 +2643,7 @@ xmlSearchNs(xmlDocPtr doc, xmlNodePtr node, const xmlChar *nameSpace) {
 	    cur = cur->next;
 	}
     }
+#endif    
     return(NULL);
 }
 
@@ -2341,6 +2661,7 @@ xmlNsPtr
 xmlSearchNsByHref(xmlDocPtr doc, xmlNodePtr node, const xmlChar *href) {
     xmlNsPtr cur;
 
+    if ((node == NULL) || (href == NULL)) return(NULL);
     while (node != NULL) {
 	cur = node->nsDef;
 	while (cur != NULL) {
@@ -2351,6 +2672,8 @@ xmlSearchNsByHref(xmlDocPtr doc, xmlNodePtr node, const xmlChar *href) {
 	}
 	node = node->parent;
     }
+#if 0    
+    /* Removed support for old namespaces */
     if (doc != NULL) {
         cur = doc->oldNs;
 	while (cur != NULL) {
@@ -2360,6 +2683,7 @@ xmlSearchNsByHref(xmlDocPtr doc, xmlNodePtr node, const xmlChar *href) {
 	    cur = cur->next;
 	}
     }
+#endif    
     return(NULL);
 }
 
@@ -2370,11 +2694,22 @@ xmlSearchNsByHref(xmlDocPtr doc, xmlNodePtr node, const xmlChar *href) {
  *
  * Search and get the value of an attribute associated to a node
  * This does the entity substitution.
+ * This function looks in DTD attribute declaration for #FIXED or
+ * default declaration values unless DTD use has been turned off.
+ *
  * Returns the attribute value or NULL if not found.
+ *     It's up to the caller to free the memory.
  */
-xmlChar *xmlGetProp(xmlNodePtr node, const xmlChar *name) {
-    xmlAttrPtr prop = node->properties;
+xmlChar *
+xmlGetProp(xmlNodePtr node, const xmlChar *name) {
+    xmlAttrPtr prop;
+    xmlDocPtr doc;
 
+    if ((node == NULL) || (name == NULL)) return(NULL);
+    /*
+     * Check on the properties attached to the node
+     */
+    prop = node->properties;
     while (prop != NULL) {
         if (!xmlStrcmp(prop->name, name))  {
 	    xmlChar *ret;
@@ -2385,6 +2720,83 @@ xmlChar *xmlGetProp(xmlNodePtr node, const xmlChar *name) {
         }
 	prop = prop->next;
     }
+    if (!xmlCheckDTD) return(NULL);
+
+    /*
+     * Check if there is a default declaration in the internal
+     * or external subsets
+     */
+    doc =  node->doc;
+    if (doc != NULL) {
+        xmlAttributePtr attrDecl;
+        if (doc->intSubset != NULL) {
+	    attrDecl = xmlGetDtdAttrDesc(doc->intSubset, node->name, name);
+	    if ((attrDecl == NULL) && (doc->extSubset != NULL))
+		attrDecl = xmlGetDtdAttrDesc(doc->extSubset, node->name, name);
+	    return(xmlStrdup(attrDecl->defaultValue));
+	}
+    }
+    return(NULL);
+}
+
+/**
+ * xmlGetNsProp:
+ * @node:  the node
+ * @name:  the attribute name
+ * @namespace:  the URI of the namespace
+ *
+ * Search and get the value of an attribute associated to a node
+ * This attribute has to be anchored in the namespace specified.
+ * This does the entity substitution.
+ * This function looks in DTD attribute declaration for #FIXED or
+ * default declaration values unless DTD use has been turned off.
+ *
+ * Returns the attribute value or NULL if not found.
+ *     It's up to the caller to free the memory.
+ */
+xmlChar *
+xmlGetNsProp(xmlNodePtr node, const xmlChar *name, const xmlChar *namespace) {
+    xmlAttrPtr prop = node->properties;
+    xmlDocPtr doc;
+    xmlNsPtr ns;
+
+    if (namespace == NULL)
+	return(xmlGetProp(node, name));
+    while (prop != NULL) {
+        if ((!xmlStrcmp(prop->name, name)) &&
+	    (prop->ns != NULL) && (!xmlStrcmp(prop->ns->href, namespace)))  {
+	    xmlChar *ret;
+
+	    ret = xmlNodeListGetString(node->doc, prop->val, 1);
+	    if (ret == NULL) return(xmlStrdup((xmlChar *)""));
+	    return(ret);
+        }
+	prop = prop->next;
+    }
+    if (!xmlCheckDTD) return(NULL);
+
+    /*
+     * Check if there is a default declaration in the internal
+     * or external subsets
+     */
+    doc =  node->doc;
+    if (doc != NULL) {
+        xmlAttributePtr attrDecl;
+        if (doc->intSubset != NULL) {
+	    attrDecl = xmlGetDtdAttrDesc(doc->intSubset, node->name, name);
+	    if ((attrDecl == NULL) && (doc->extSubset != NULL))
+		attrDecl = xmlGetDtdAttrDesc(doc->extSubset, node->name, name);
+		
+	    if (attrDecl->prefix != NULL) {
+	        /*
+		 * The DTD declaration only allows a prefix search
+		 */
+		ns = xmlSearchNs(doc, node, attrDecl->prefix);
+		if ((ns != NULL) && (!xmlStrcmp(ns->href, namespace)))
+		    return(xmlStrdup(attrDecl->defaultValue));
+	    }
+	}
+    }
     return(NULL);
 }
 
@@ -2406,8 +2818,12 @@ xmlSetProp(xmlNodePtr node, const xmlChar *name, const xmlChar *value) {
 	    if (prop->val != NULL) 
 	        xmlFreeNodeList(prop->val);
 	    prop->val = NULL;
-	    if (value != NULL)
-		prop->val = xmlStringGetNodeList(node->doc, value);
+	    if (value != NULL) {
+	        xmlChar *buffer;
+		buffer = xmlEncodeEntitiesReentrant(node->doc, value);
+		prop->val = xmlStringGetNodeList(node->doc, buffer);
+		xmlFree(buffer);
+	    }	
 	    return(prop);
 	}
 	prop = prop->next;
@@ -2448,7 +2864,11 @@ xmlTextConcat(xmlNodePtr node, const xmlChar *content, int len) {
 	fprintf(stderr, "xmlTextConcat: node is not text\n");
         return;
     }
+#ifndef XML_USE_BUFFER_CONTENT
     node->content = xmlStrncat(node->content, content, len);
+#else
+    xmlBufferAdd(node->content, content, len);
+#endif
 }
 
 /************************************************************************
@@ -2476,6 +2896,7 @@ xmlBufferCreate(void) {
     }
     ret->use = 0;
     ret->size = BASE_BUFFER_SIZE;
+    ret->alloc = xmlBufferAllocScheme;
     ret->content = (xmlChar *) xmlMalloc(ret->size * sizeof(xmlChar));
     if (ret->content == NULL) {
 	fprintf(stderr, "xmlBufferCreate : out of memory!\n");
@@ -2486,6 +2907,56 @@ xmlBufferCreate(void) {
     return(ret);
 }
 
+/**
+ * xmlBufferCreateSize:
+ * @size: initial size of buffer
+ *
+ * routine to create an XML buffer.
+ * returns the new structure.
+ */
+xmlBufferPtr
+xmlBufferCreateSize(size_t size) {
+    xmlBufferPtr ret;
+
+    ret = (xmlBufferPtr) xmlMalloc(sizeof(xmlBuffer));
+    if (ret == NULL) {
+        fprintf(stderr, "xmlBufferCreate : out of memory!\n");
+        return(NULL);
+    }
+    ret->use = 0;
+    ret->alloc = xmlBufferAllocScheme;
+    ret->size = (size ? size+2 : 0);         /* +1 for ending null */
+    if (ret->size){
+        ret->content = (xmlChar *) xmlMalloc(ret->size * sizeof(xmlChar));
+        if (ret->content == NULL) {
+            fprintf(stderr, "xmlBufferCreate : out of memory!\n");
+            xmlFree(ret);
+            return(NULL);
+        }
+        ret->content[0] = 0;
+    } else
+	ret->content = NULL;
+    return(ret);
+}
+
+/**
+ * xmlBufferAllocationScheme:
+ * @buf:  the buffer to free
+ * @scheme:  allocation scheme to use
+ *
+ * Sets the allocation scheme for this buffer
+ */
+void
+xmlBufferSetAllocationScheme(xmlBufferPtr buf, 
+                             xmlBufferAllocationScheme scheme) {
+    if (buf == NULL) {
+        fprintf(stderr, "xmlBufferSetAllocationScheme: buf == NULL\n");
+        return;
+    }
+
+    buf->alloc = scheme;
+}
+
 /**
  * xmlBufferFree:
  * @buf:  the buffer to free
@@ -2498,10 +2969,12 @@ xmlBufferFree(xmlBufferPtr buf) {
         fprintf(stderr, "xmlBufferFree: buf == NULL\n");
 	return;
     }
-    if (buf->content == NULL) {
-        fprintf(stderr, "xmlBufferFree: buf->content == NULL\n");
-    } else {
+    if (buf->content != NULL) {
+#ifndef XML_USE_BUFFER_CONTENT
         memset(buf->content, -1, BASE_BUFFER_SIZE);
+#else
+        memset(buf->content, -1, buf->size);
+#endif
         xmlFree(buf->content);
     }
     memset(buf, -1, sizeof(xmlBuffer));
@@ -2566,40 +3039,126 @@ xmlBufferDump(FILE *file, xmlBufferPtr buf) {
     return(ret);
 }
 
+/**
+ * xmlBufferContent:
+ * @buf:  the buffer to resize
+ *
+ * Returns the internal content
+ */
+
+const xmlChar* 
+xmlBufferContent(const xmlBufferPtr buf)
+{
+    if(!buf)
+        return NULL;
+
+    return buf->content;
+}
+
+/**
+ * xmlBufferLength:
+ * @buf:  the buffer 
+ *
+ * Returns the length of data in the internal content
+ */
+
+int
+xmlBufferLength(const xmlBufferPtr buf)
+{
+    if(!buf)
+        return 0;
+
+    return buf->use;
+}
+
+/**
+ * xmlBufferResize:
+ * @buf:  the buffer to resize
+ * @len:  the desired size
+ *
+ * Resize a buffer to accomodate minimum size of <len>.
+ *
+ * Returns  0 in case of problems, 1 otherwise
+ */
+int
+xmlBufferResize(xmlBufferPtr buf, int size)
+{
+    int newSize = (buf->size ? buf->size*2 : size);/*take care of empty case*/
+    xmlChar* rebuf = NULL;
+
+    /* Don't resize if we don't have to */
+    if(size < buf->size)
+        return 1;
+
+    /* figure out new size */
+    switch(buf->alloc){
+    case XML_BUFFER_ALLOC_DOUBLEIT:
+        while(size > newSize) newSize *= 2;
+        break;
+    case XML_BUFFER_ALLOC_EXACT:
+        newSize = size+10;
+        break;
+    default:
+        newSize = size+10;
+        break;
+    }
+
+    if (buf->content == NULL)
+	rebuf = (xmlChar *) xmlMalloc(newSize * sizeof(xmlChar));
+    else
+	rebuf = (xmlChar *) xmlRealloc(buf->content, 
+				       newSize * sizeof(xmlChar));
+    if (rebuf == NULL) {
+        fprintf(stderr, "xmlBufferAdd : out of memory!\n");
+        return 0;
+    }
+    buf->content = rebuf;
+    buf->size = newSize;
+
+    return 1;
+}
 /**
  * xmlBufferAdd:
  * @buf:  the buffer to dump
  * @str:  the xmlChar string
  * @len:  the number of xmlChar to add
  *
- * Add a string range to an XML buffer.
+ * Add a string range to an XML buffer. if len == -1, the lenght of
+ * str is recomputed.
  */
 void
 xmlBufferAdd(xmlBufferPtr buf, const xmlChar *str, int len) {
-    int l;
+    int l, needSize;
 
     if (str == NULL) {
         fprintf(stderr, "xmlBufferAdd: str == NULL\n");
 	return;
     }
+    if (len < -1) {
+        fprintf(stderr, "xmlBufferAdd: len < 0\n");
+	return;
+    }
+    if (len == 0) return;
+
+    /* CJN What's this for??? */
     l = xmlStrlen(str);
-    if (l < len) len = l;
+    if (l < len){  len = l; printf("xmlBufferAdd bad length\n"); }
+
+    /* CJN 11.18.99 okay, now I'm using the length */
+    if(len == -1) len = l;
+
+
     if (len <= 0) return;
 
-    if (buf->use + len + 10 >= buf->size) {
-	xmlChar *rebuf;
-
-        buf->size *= 2;
-	if (buf->use + len + 10 > buf->size)
-	    buf->size = buf->use + len + 10;
-	rebuf = (xmlChar *) xmlRealloc(buf->content, buf->size * sizeof(xmlChar));
-	if (rebuf == NULL) {
-	    fprintf(stderr, "xmlBufferAdd : out of memory!\n");
-	    return;
-	}
-	buf->content = rebuf;
+    needSize = buf->use + len + 2;
+    if(needSize > buf->size){
+        if(!xmlBufferResize(buf, needSize)){
+            fprintf(stderr, "xmlBufferAdd : out of memory!\n");
+            return;
+        }
     }
-    memmove(&buf->content[buf->use], str, len);
+
+    memmove(&buf->content[buf->use], str, len*sizeof(xmlChar));
     buf->use += len;
     buf->content[buf->use] = 0;
 }
@@ -2613,26 +3172,8 @@ xmlBufferAdd(xmlBufferPtr buf, const xmlChar *str, int len) {
  */
 void
 xmlBufferCat(xmlBufferPtr buf, const xmlChar *str) {
-    const xmlChar *cur;
-
-    if (str == NULL) {
-        fprintf(stderr, "xmlBufferAdd: str == NULL\n");
-	return;
-    }
-    for (cur = str;*cur != 0;cur++) {
-        if (buf->use  + 10 >= buf->size) {
-	    xmlChar *rebuf;
-
-	    buf->size *= 2;
-	    rebuf = (xmlChar *) xmlRealloc(buf->content, buf->size * sizeof(xmlChar));
-	    if (rebuf == NULL) {
-	        fprintf(stderr, "xmlBufferAdd : out of memory!\n");
-		return;
-	    }
-	    buf->content = rebuf;
-	}
-        buf->content[buf->use++] = *cur;
-    }
+    if (str != NULL)
+	xmlBufferAdd(buf, str, -1);
 }
 
 /**
@@ -2652,34 +3193,15 @@ xmlBufferCCat(xmlBufferPtr buf, const char *str) {
     }
     for (cur = str;*cur != 0;cur++) {
         if (buf->use  + 10 >= buf->size) {
-	    xmlChar *rebuf;
-
-	    buf->size *= 2;
-	    rebuf = (xmlChar *) xmlRealloc(buf->content, buf->size * sizeof(xmlChar));
-	    if (rebuf == NULL) {
-	        fprintf(stderr, "xmlBufferAdd : out of memory!\n");
-		return;
-	    }
-	    buf->content = rebuf;
-	}
+            if(!xmlBufferResize(buf, buf->use+10)){
+                fprintf(stderr, "xmlBufferCCat : out of memory!\n");
+                return;
+            }
+        }
         buf->content[buf->use++] = *cur;
     }
 }
 
-/**
- * xmlBufferLastChar:
- * @buf:  the buffer to dump
- *
- * Get the last char of the buffer
- *
- * Returns the last char from the buffer or 0 if empty
- */
-xmlChar
-xmlBufferLastChar(xmlBufferPtr buf) {
-    if ((buf == NULL) || (buf->use <= 0)) return(0);
-    return(buf->content[buf->use - 1]);
-}
-
 /**
  * xmlBufferWriteCHAR:
  * @buf:  the XML buffer
@@ -2975,7 +3497,12 @@ xmlNodeDump(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur, int level,
 	if (cur->content != NULL) {
             xmlChar *buffer;
 
+#ifndef XML_USE_BUFFER_CONTENT
             buffer = xmlEncodeEntitiesReentrant(doc, cur->content);
+#else
+	    buffer = xmlEncodeEntitiesReentrant(doc, 
+					    xmlBufferContent(cur->content));
+#endif
 	    if (buffer != NULL) {
 		xmlBufferWriteCHAR(buf, buffer);
 		xmlFree(buffer);
@@ -2989,7 +3516,11 @@ xmlNodeDump(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur, int level,
 	    xmlBufferWriteCHAR(buf, cur->name);
 	    if (cur->content != NULL) {
 		xmlBufferWriteChar(buf, " ");
+#ifndef XML_USE_BUFFER_CONTENT
 		xmlBufferWriteCHAR(buf, cur->content);
+#else
+		xmlBufferWriteCHAR(buf, xmlBufferContent(cur->content));
+#endif
 	    }
 	    xmlBufferWriteChar(buf, "?>");
 	}
@@ -2998,7 +3529,11 @@ xmlNodeDump(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur, int level,
     if (cur->type == XML_COMMENT_NODE) {
 	if (cur->content != NULL) {
 	    xmlBufferWriteChar(buf, "<!--");
+#ifndef XML_USE_BUFFER_CONTENT
 	    xmlBufferWriteCHAR(buf, cur->content);
+#else
+	    xmlBufferWriteCHAR(buf, xmlBufferContent(cur->content));
+#endif
 	    xmlBufferWriteChar(buf, "-->");
 	}
 	return;
@@ -3012,7 +3547,11 @@ xmlNodeDump(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur, int level,
     if (cur->type == XML_CDATA_SECTION_NODE) {
         xmlBufferWriteChar(buf, "<![CDATA[");
 	if (cur->content != NULL)
+#ifndef XML_USE_BUFFER_CONTENT
 	    xmlBufferWriteCHAR(buf, cur->content);
+#else
+	    xmlBufferWriteCHAR(buf, xmlBufferContent(cur->content));
+#endif
         xmlBufferWriteChar(buf, "]]>");
 	return;
     }
@@ -3048,7 +3587,12 @@ xmlNodeDump(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur, int level,
     if (cur->content != NULL) {
 	xmlChar *buffer;
 
+#ifndef XML_USE_BUFFER_CONTENT
 	buffer = xmlEncodeEntitiesReentrant(doc, cur->content);
+#else
+	buffer = xmlEncodeEntitiesReentrant(doc, 
+		                            xmlBufferContent(cur->content));
+#endif
 	if (buffer != NULL) {
 	    xmlBufferWriteCHAR(buf, buffer);
 	    xmlFree(buffer);
@@ -3234,7 +3778,8 @@ xmlDocDump(FILE *f, xmlDocPtr cur) {
  * @cur:  the document
  *
  * Dump an XML document to a file. Will use compression if
- * compiled in and enabled.
+ * compiled in and enabled. If @filename is "-" the stdout file is
+ * used.
  * returns: the number of file written or -1 in case of failure.
  */
 int
@@ -3255,14 +3800,21 @@ xmlSaveFile(const char *filename, xmlDocPtr cur) {
     xmlDocContentDump(buf, cur);
 
 #ifdef HAVE_ZLIB_H
+    if (cur->compression < 0) cur->compression = xmlCompressMode;
     if ((cur->compression > 0) && (cur->compression <= 9)) {
         sprintf(mode, "w%d", cur->compression);
-	zoutput = gzopen(filename, mode);
+	if (!strcmp(filename, "-")) 
+	    zoutput = gzdopen(1, mode);
+	else
+	    zoutput = gzopen(filename, mode);
     }
     if (zoutput == NULL) {
 #endif
         output = fopen(filename, "w");
-	if (output == NULL) return(-1);
+	if (output == NULL) {
+	    xmlBufferFree(buf);
+	    return(-1);
+	}
 #ifdef HAVE_ZLIB_H
     }
 
