@@ -309,7 +309,7 @@ xmlCreateElementTable(void) {
  */
 xmlElementPtr
 xmlAddElementDecl(xmlValidCtxtPtr ctxt, xmlDtdPtr dtd, const xmlChar *name,
-                  xmlElementContentType type, xmlElementContentPtr content) {
+                  xmlElementTypeVal type, xmlElementContentPtr content) {
     xmlElementPtr ret, cur;
     xmlElementTablePtr table;
     int i;
@@ -760,6 +760,8 @@ xmlAddAttributeDecl(xmlValidCtxtPtr ctxt, xmlDtdPtr dtd, const xmlChar *elem,
     xmlAttributePtr ret, cur;
     xmlAttributeTablePtr table;
     xmlElementPtr elemDef;
+    xmlChar *rname;
+    xmlChar *ns;
     int i;
 
     if (dtd == NULL) {
@@ -820,13 +822,21 @@ xmlAddAttributeDecl(xmlValidCtxtPtr ctxt, xmlDtdPtr dtd, const xmlChar *elem,
         return(NULL);
     }
 
+    /*
+     * Split the full name into a namespace prefix and the tag name
+     */
+    rname = xmlSplitQName(name, &ns);
+
     /*
      * Validity Check:
      * Search the DTD for previous declarations of the ATTLIST
      */
     for (i = 0;i < table->nb_attributes;i++) {
         cur = table->table[i];
-	if ((!xmlStrcmp(cur->name, name)) && (!xmlStrcmp(cur->elem, elem))) {
+	if ((ns != NULL) && (cur->prefix == NULL)) continue;
+	if ((ns == NULL) && (cur->prefix != NULL)) continue;
+	if ((!xmlStrcmp(cur->name, rname)) && (!xmlStrcmp(cur->elem, elem)) &&
+	    ((ns == NULL) || (!xmlStrcmp(cur->prefix, ns)))) {
 	    /*
 	     * The attribute is already defined in this Dtd.
 	     */
@@ -862,7 +872,8 @@ xmlAddAttributeDecl(xmlValidCtxtPtr ctxt, xmlDtdPtr dtd, const xmlChar *elem,
      * fill the structure.
      */
     ret->type = type;
-    ret->name = xmlStrdup(name);
+    ret->name = rname;
+    ret->prefix = ns;
     ret->elem = xmlStrdup(elem);
     ret->def = def;
     ret->tree = tree;
@@ -902,6 +913,8 @@ xmlFreeAttribute(xmlAttributePtr attr) {
 	xmlFree((xmlChar *) attr->name);
     if (attr->defaultValue != NULL)
 	xmlFree((xmlChar *) attr->defaultValue);
+    if (attr->prefix != NULL)
+	xmlFree((xmlChar *) attr->prefix);
     memset(attr, -1, sizeof(xmlAttribute));
     xmlFree(attr);
 }
@@ -2059,7 +2072,6 @@ xmlValidateNmtokenValue(const xmlChar *value) {
     if (*cur != 0) return(0);
 
     return(1);
-    return(1);
 }
 
 /**
@@ -2994,8 +3006,8 @@ xmlValidGetPotentialChildren(xmlElementContent *ctree, const xmlChar **list,
     switch (ctree->type) {
 	case XML_ELEMENT_CONTENT_PCDATA: 
 	    for (i = 0; i < *len;i++)
-		if (!xmlStrcmp("#PCDATA", list[i])) return(*len);
-	    list[(*len)++] = "#PCDATA";
+		if (!xmlStrcmp(BAD_CAST "#PCDATA", list[i])) return(*len);
+	    list[(*len)++] = BAD_CAST "#PCDATA";
 	    break;
 	case XML_ELEMENT_CONTENT_ELEMENT: 
 	    for (i = 0; i < *len;i++)
@@ -3089,7 +3101,7 @@ xmlValidGetValidElements(xmlNode *prev, xmlNode *next, const xmlChar **list,
     /*
      * Creates a dummy node and insert it into the tree
      */    
-    test_node = xmlNewNode (NULL, "<!dummy?>");
+    test_node = xmlNewNode (NULL, BAD_CAST "<!dummy?>");
     test_node->doc = ref_node->doc;
     test_node->parent = parent;
     test_node->prev = prev;
